@@ -4,7 +4,7 @@
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -77,7 +77,7 @@ mmcli_get_manager (GDBusConnection *connection,
                                         user_data,
                                         mmcli_get_manager);
     mm_manager_new (connection,
-                    G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_NONE,
+                    G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_DO_NOT_AUTO_START,
                     cancellable,
                     (GAsyncReadyCallback)manager_new_ready,
                     result);
@@ -91,7 +91,7 @@ mmcli_get_manager_sync (GDBusConnection *connection)
     GError *error = NULL;
 
     manager = mm_manager_new_sync (connection,
-                                   G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_NONE,
+                                   G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_DO_NOT_AUTO_START,
                                    NULL,
                                    &error);
     if (!manager) {
@@ -476,7 +476,7 @@ mmcli_get_bearer (GDBusConnection *connection,
     ctx->result = g_simple_async_result_new (G_OBJECT (connection),
                                              callback,
                                              user_data,
-                                             mmcli_get_modem);
+                                             mmcli_get_bearer);
     mmcli_get_manager (connection,
                        cancellable,
                        (GAsyncReadyCallback)get_bearer_manager_ready,
@@ -714,7 +714,7 @@ mmcli_get_sim (GDBusConnection *connection,
     ctx->result = g_simple_async_result_new (G_OBJECT (connection),
                                              callback,
                                              user_data,
-                                             mmcli_get_modem);
+                                             mmcli_get_sim);
     mmcli_get_manager (connection,
                        cancellable,
                        (GAsyncReadyCallback)get_sim_manager_ready,
@@ -977,7 +977,7 @@ mmcli_get_sms (GDBusConnection *connection,
     ctx->result = g_simple_async_result_new (G_OBJECT (connection),
                                              callback,
                                              user_data,
-                                             mmcli_get_modem);
+                                             mmcli_get_sms);
     mmcli_get_manager (connection,
                        cancellable,
                        (GAsyncReadyCallback)get_sms_manager_ready,
@@ -1063,6 +1063,8 @@ mmcli_get_state_reason_string (MMModemStateChangeReason reason)
         return "User request";
     case MM_MODEM_STATE_CHANGE_REASON_SUSPEND:
         return "Suspend";
+    case MM_MODEM_STATE_CHANGE_REASON_FAILURE:
+        return "Failure";
     }
 
     g_warn_if_reached ();
@@ -1098,17 +1100,17 @@ static GOptionEntry entries[] = {
 GOptionGroup *
 mmcli_get_common_option_group (void)
 {
-	GOptionGroup *group;
+    GOptionGroup *group;
 
-	/* Status options */
-	group = g_option_group_new ("common",
-	                            "Common options",
-	                            "Show common options",
-	                            NULL,
-	                            NULL);
-	g_option_group_add_entries (group, entries);
+    /* Status options */
+    group = g_option_group_new ("common",
+                                "Common options",
+                                "Show common options",
+                                NULL,
+                                NULL);
+    g_option_group_add_entries (group, entries);
 
-	return group;
+    return group;
 }
 
 const gchar *
@@ -1143,10 +1145,15 @@ mmcli_prefix_newlines (const gchar *prefix,
     const gchar *line_start = str;
     const gchar *line_end;
 
-    while ((line_end = strchr (line_start, '\n'))) {
+    do {
         gssize line_length;
 
-        line_length = line_end - line_start;
+        line_end = strchr (line_start, '\n');
+        if (line_end)
+            line_length = line_end - line_start;
+        else
+            line_length = strlen (line_start);
+
         if (line_start[line_length - 1] == '\r')
             line_length--;
 
@@ -1164,8 +1171,8 @@ mmcli_prefix_newlines (const gchar *prefix,
                                  line_length);
         }
 
-        line_start = line_end + 1;
-    }
+        line_start = (line_end ? line_end + 1 : NULL);
+    } while (line_start != NULL);
 
     return (prefixed_string ?
             g_string_free (prefixed_string, FALSE) :
