@@ -87,6 +87,9 @@ guint              mm_modem_get_max_bearers          (MMModem *self);
 
 guint              mm_modem_get_max_active_bearers   (MMModem *self);
 
+const gchar * const *mm_modem_get_bearer_paths       (MMModem *self);
+gchar              **mm_modem_dup_bearer_paths       (MMModem *self);
+
 const gchar       *mm_modem_get_manufacturer         (MMModem *self);
 gchar             *mm_modem_dup_manufacturer         (MMModem *self);
 
@@ -111,6 +114,13 @@ gchar             *mm_modem_dup_plugin               (MMModem *self);
 const gchar       *mm_modem_get_primary_port         (MMModem *self);
 gchar             *mm_modem_dup_primary_port         (MMModem *self);
 
+gboolean           mm_modem_peek_ports               (MMModem *self,
+                                                      const MMModemPortInfo **ports,
+                                                      guint *n_ports);
+gboolean           mm_modem_get_ports                (MMModem *self,
+                                                      MMModemPortInfo **ports,
+                                                      guint *n_ports);
+
 const gchar       *mm_modem_get_equipment_identifier (MMModem *self);
 gchar             *mm_modem_dup_equipment_identifier (MMModem *self);
 
