@@ -23,7 +23,7 @@
 #define _LIBMM_INSIDE_MM
 #include <libmm-glib.h>
 
-#include "mm-serial-enums-types.h"
+#include "mm-port-enums-types.h"
 #include "mm-log.h"
 #include "mm-plugin-huawei.h"
 #include "mm-broadband-modem-huawei.h"
@@ -32,6 +32,10 @@
 #include "mm-broadband-modem-qmi.h"
 #endif
 
+#if defined WITH_MBIM
+#include "mm-broadband-modem-mbim.h"
+#endif
+
 G_DEFINE_TYPE (MMPluginHuawei, mm_plugin_huawei, MM_TYPE_PLUGIN)
 
 int mm_plugin_major_version = MM_PLUGIN_MAJOR_VERSION;
@@ -70,7 +74,7 @@ first_interface_context_free (FirstInterfaceContext *ctx)
 
 typedef struct {
     MMPortProbe *probe;
-    MMAtSerialPort *port;
+    MMPortSerialAt *port;
     GCancellable *cancellable;
     GSimpleAsyncResult *result;
     gboolean curc_done;
@@ -123,11 +127,14 @@ cache_port_mode (MMDevice *device,
 }
 
 static void
-getportmode_ready (MMAtSerialPort *port,
-                   GString *response,
-                   GError *error,
+getportmode_ready (MMPortSerialAt *port,
+                   GAsyncResult *res,
                    HuaweiCustomInitContext *ctx)
 {
+    const gchar *response;
+    GError *error = NULL;
+
+    response = mm_port_serial_at_command_finish (port, res, &error);
     if (error) {
         mm_dbg ("(Huawei) couldn't get port mode: '%s'",
                 error->message);
@@ -137,11 +144,8 @@ getportmode_ready (MMAtSerialPort *port,
          */
         if (!g_error_matches (error,
                               MM_MOBILE_EQUIPMENT_ERROR,
-                              MM_MOBILE_EQUIPMENT_ERROR_UNKNOWN)) {
-            /* Retry */
-            huawei_custom_init_step (ctx);
-            return;
-        }
+                              MM_MOBILE_EQUIPMENT_ERROR_UNKNOWN))
+            goto out;
 
         /* Port mode not supported */
     } else {
@@ -151,10 +155,14 @@ getportmode_ready (MMAtSerialPort *port,
 
         /* Results are cached in the parent device object */
         device = mm_port_probe_peek_device (ctx->probe);
-        cache_port_mode (device, response->str, "PCUI:", TAG_HUAWEI_PCUI_PORT);
-        cache_port_mode (device, response->str, "MDM:",  TAG_HUAWEI_MODEM_PORT);
-        cache_port_mode (device, response->str, "NDIS:", TAG_HUAWEI_NDIS_PORT);
-        cache_port_mode (device, response->str, "DIAG:", TAG_HUAWEI_DIAG_PORT);
+        cache_port_mode (device, response, "PCUI:", TAG_HUAWEI_PCUI_PORT);
+        cache_port_mode (device, response, "MDM:",  TAG_HUAWEI_MODEM_PORT);
+        cache_port_mode (device, response, "NDIS:", TAG_HUAWEI_NDIS_PORT);
+        cache_port_mode (device, response, "DIAG:", TAG_HUAWEI_DIAG_PORT);
+        /* GETPORTMODE response format in newer devices... (e.g. E3372) */
+        cache_port_mode (device, response, "pcui:",  TAG_HUAWEI_PCUI_PORT);
+        cache_port_mode (device, response, "modem:", TAG_HUAWEI_MODEM_PORT);
+
         g_object_set_data (G_OBJECT (device), TAG_GETPORTMODE_SUPPORTED, GUINT_TO_POINTER (TRUE));
 
         /* Mark port as being AT already */
@@ -162,48 +170,97 @@ getportmode_ready (MMAtSerialPort *port,
     }
 
     ctx->getportmode_done = TRUE;
+
+out:
+    if (error)
+        g_error_free (error);
+
     huawei_custom_init_step (ctx);
 }
 
 static void
-curc_ready (MMAtSerialPort *port,
-            GString *response,
-            GError *error,
+curc_ready (MMPortSerialAt *port,
+            GAsyncResult *res,
             HuaweiCustomInitContext *ctx)
 {
+    const gchar *response;
+    GError *error = NULL;
+
+    response = mm_port_serial_at_command_finish (port, res, &error);
     if (error) {
         /* Retry if we get a timeout error */
         if (g_error_matches (error,
                              MM_SERIAL_ERROR,
-                             MM_SERIAL_ERROR_RESPONSE_TIMEOUT)) {
-            huawei_custom_init_step (ctx);
-            return;
-        }
+                             MM_SERIAL_ERROR_RESPONSE_TIMEOUT))
+            goto out;
 
-        mm_dbg ("(Huawei) couldn't turn off unsolicited messages in"
-                "secondary ports: '%s'",
+        mm_dbg ("(Huawei) couldn't turn off unsolicited messages in secondary ports: '%s'",
                 error->message);
     }
 
     mm_dbg ("(Huawei) unsolicited messages in secondary ports turned off");
 
     ctx->curc_done = TRUE;
+
+out:
+    if (error)
+        g_error_free (error);
+
     huawei_custom_init_step (ctx);
 }
 
+static void
+try_next_usbif (MMDevice *device)
+{
+    FirstInterfaceContext *fi_ctx;
+    GList *l;
+    gint closest;
+
+    fi_ctx = g_object_get_data (G_OBJECT (device), TAG_FIRST_INTERFACE_CONTEXT);
+    g_assert (fi_ctx != NULL);
+
+    /* Look for the next closest one among the list of interfaces in the device,
+     * and enable that one as being first */
+    closest = G_MAXINT;
+    for (l = mm_device_peek_port_probe_list (device); l; l = g_list_next (l)) {
+        MMPortProbe *probe = MM_PORT_PROBE (l->data);
+
+        /* Only expect ttys for next probing attempt */
+        if (g_str_equal (mm_port_probe_get_port_subsys (probe), "tty")) {
+            gint usbif;
+
+            usbif = g_udev_device_get_property_as_int (mm_port_probe_peek_port (probe), "ID_USB_INTERFACE_NUM");
+            if (usbif == fi_ctx->first_usbif) {
+                /* This is the one we just probed, which wasn't yet removed, so just skip it */
+            } else if (usbif > fi_ctx->first_usbif &&
+                       usbif < closest) {
+                closest = usbif;
+            }
+        }
+    }
+
+    if (closest == G_MAXINT) {
+        /* No more ttys to try! Just return something */
+        closest = 0;
+        mm_dbg ("(Huawei) No more ports to run initial probing");
+    } else {
+        mm_dbg ("(Huawei) Will try initial probing with interface '%d' instead", closest);
+    }
+
+    fi_ctx->first_usbif = closest;
+}
+
 static void
 huawei_custom_init_step (HuaweiCustomInitContext *ctx)
 {
     FirstInterfaceContext *fi_ctx;
+    GUdevDevice *port;
 
     /* If cancelled, end */
     if (g_cancellable_is_cancelled (ctx->cancellable)) {
         mm_dbg ("(Huawei) no need to keep on running custom init in (%s)",
                 mm_port_get_device (MM_PORT (ctx->port)));
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_CANCELLED,
-                                         "Custom initialization cancelled");
+        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
         huawei_custom_init_context_complete_and_free (ctx);
         return;
     }
@@ -213,25 +270,29 @@ huawei_custom_init_step (HuaweiCustomInitContext *ctx)
             /* All retries consumed, probably not an AT port */
             mm_port_probe_set_result_at (ctx->probe, FALSE);
             g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+            /* Try with next */
+            try_next_usbif (mm_port_probe_peek_device (ctx->probe));
             huawei_custom_init_context_complete_and_free (ctx);
             return;
         }
 
         ctx->curc_retries--;
         /* Turn off unsolicited messages on secondary ports until needed */
-        mm_at_serial_port_queue_command (
+        mm_port_serial_at_command (
             ctx->port,
             "AT^CURC=0",
             3,
             FALSE, /* raw */
+            FALSE, /* allow_cached */
             ctx->cancellable,
-            (MMAtSerialResponseFn)curc_ready,
+            (GAsyncReadyCallback)curc_ready,
             ctx);
         return;
     }
 
     /* Try to get a port map from the modem */
-    if (!ctx->getportmode_done) {
+    port = mm_port_probe_peek_port (ctx->probe);
+    if (!ctx->getportmode_done && !g_udev_device_get_property_as_boolean (port, "ID_MM_HUAWEI_DISABLE_GETPORTMODE")) {
         if (ctx->getportmode_retries == 0) {
             g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
             huawei_custom_init_context_complete_and_free (ctx);
@@ -239,13 +300,14 @@ huawei_custom_init_step (HuaweiCustomInitContext *ctx)
         }
 
         ctx->getportmode_retries--;
-        mm_at_serial_port_queue_command (
+        mm_port_serial_at_command (
             ctx->port,
             "AT^GETPORTMODE",
             3,
             FALSE, /* raw */
+            FALSE, /* allow_cached */
             ctx->cancellable,
-            (MMAtSerialResponseFn)getportmode_ready,
+            (GAsyncReadyCallback)getportmode_ready,
             ctx);
         return;
     }
@@ -262,37 +324,7 @@ huawei_custom_init_step (HuaweiCustomInitContext *ctx)
 static gboolean
 first_interface_missing_timeout_cb (MMDevice *device)
 {
-    FirstInterfaceContext *fi_ctx;
-    GList *l;
-    gint closest;
-
-    fi_ctx = g_object_get_data (G_OBJECT (device), TAG_FIRST_INTERFACE_CONTEXT);
-    g_assert (fi_ctx != NULL);
-
-    /* First interface didn't appear, look for the next closest one among the list of
-     * interfaces in the device, and enable that one as being first */
-    closest = G_MAXINT;
-    for (l = mm_device_peek_port_probe_list (device); l; l = g_list_next (l)) {
-        gint usbif;
-
-        usbif = g_udev_device_get_property_as_int (mm_port_probe_peek_port (MM_PORT_PROBE (l->data)), "ID_USB_INTERFACE_NUM");
-        if (usbif == fi_ctx->first_usbif) {
-            g_warn_if_reached ();
-        } else if (usbif > fi_ctx->first_usbif &&
-                   usbif < closest) {
-            closest = usbif;
-        }
-    }
-
-    if (closest == G_MAXINT) {
-        /* Retry with interface 0... */
-        closest = 0;
-    }
-
-    mm_dbg ("(Huawei) Couldn't find interface '%d' to start probing, will try with interface '%d' first instead",
-            fi_ctx->first_usbif, closest);
-
-    fi_ctx->first_usbif = closest;
+    try_next_usbif (device);
 
     /* Reload the timeout, just in case we end up not having the next interface to probe...
      * which is anyway very unlikely as we got it by looking at the real probe list, but anyway... */
@@ -301,7 +333,7 @@ first_interface_missing_timeout_cb (MMDevice *device)
 
 static void
 huawei_custom_init (MMPortProbe *probe,
-                    MMAtSerialPort *port,
+                    MMPortSerialAt *port,
                     GCancellable *cancellable,
                     GAsyncReadyCallback callback,
                     gpointer user_data)
@@ -376,8 +408,10 @@ huawei_custom_init (MMPortProbe *probe,
     }
 
     /* We can run custom init in the first interface! clear the timeout as it is no longer needed */
-    g_source_remove (fi_ctx->timeout_id);
-    fi_ctx->timeout_id = 0;
+    if (fi_ctx->timeout_id) {
+        g_source_remove (fi_ctx->timeout_id);
+        fi_ctx->timeout_id = 0;
+    }
 
     huawei_custom_init_step (ctx);
 }
@@ -389,41 +423,60 @@ propagate_port_mode_results (GList *probes)
 {
     MMDevice *device;
     GList *l;
+    gboolean primary_flagged = FALSE;
 
     g_assert (probes != NULL);
     device = mm_port_probe_peek_device (MM_PORT_PROBE (probes->data));
 
     /* Now we propagate the tags to the specific port probes */
     for (l = probes; l; l = g_list_next (l)) {
-        MMAtPortFlag at_port_flags = MM_AT_PORT_FLAG_NONE;
+        MMPortSerialAtFlag at_port_flags = MM_PORT_SERIAL_AT_FLAG_NONE;
         gint usbif;
 
         usbif = g_udev_device_get_property_as_int (mm_port_probe_peek_port (MM_PORT_PROBE (l->data)), "ID_USB_INTERFACE_NUM");
 
         if (GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (device), TAG_GETPORTMODE_SUPPORTED))) {
-            if (usbif + 1 == GPOINTER_TO_INT (g_object_get_data (G_OBJECT (device), TAG_HUAWEI_PCUI_PORT)))
-                at_port_flags = MM_AT_PORT_FLAG_PRIMARY;
-            else if (usbif + 1 == GPOINTER_TO_INT (g_object_get_data (G_OBJECT (device), TAG_HUAWEI_MODEM_PORT)))
-                at_port_flags = MM_AT_PORT_FLAG_PPP;
+            if (usbif + 1 == GPOINTER_TO_INT (g_object_get_data (G_OBJECT (device), TAG_HUAWEI_PCUI_PORT))) {
+                at_port_flags = MM_PORT_SERIAL_AT_FLAG_PRIMARY;
+                primary_flagged = TRUE;
+            } else if (usbif + 1 == GPOINTER_TO_INT (g_object_get_data (G_OBJECT (device), TAG_HUAWEI_MODEM_PORT)))
+                at_port_flags = MM_PORT_SERIAL_AT_FLAG_PPP;
             else if (!g_object_get_data (G_OBJECT (device), TAG_HUAWEI_MODEM_PORT) &&
                      usbif + 1 == GPOINTER_TO_INT (g_object_get_data (G_OBJECT (device), TAG_HUAWEI_NDIS_PORT)))
                 /* If NDIS reported only instead of MDM, use it */
-                at_port_flags = MM_AT_PORT_FLAG_PPP;
+                at_port_flags = MM_PORT_SERIAL_AT_FLAG_PPP;
         } else if (usbif == 0 &&
                    mm_port_probe_is_at (MM_PORT_PROBE (l->data))) {
             /* If GETPORTMODE is not supported, we assume usbif 0 is the modem port */
-            at_port_flags = MM_AT_PORT_FLAG_PPP;
+            at_port_flags = MM_PORT_SERIAL_AT_FLAG_PPP;
 
             /* /\* TODO. */
             /*  * For CDMA modems we assume usbif0 is both primary and PPP, since */
             /*  * they don't have problems with talking on secondary ports. */
             /*  *\/ */
             /* if (caps & CAP_CDMA) */
-            /*     pflags |= MM_AT_PORT_FLAG_PRIMARY; */
+            /*     pflags |= MM_PORT_SERIAL_AT_FLAG_PRIMARY; */
         }
 
         g_object_set_data (G_OBJECT (l->data), TAG_AT_PORT_FLAGS, GUINT_TO_POINTER (at_port_flags));
     }
+
+    if (primary_flagged)
+        return;
+
+    /* For devices exposing a cdc-wdm port, make sure it gets flagged as primary, if there is none
+     * already */
+    for (l = probes; l; l = g_list_next (l)) {
+        MMPortProbe *probe = MM_PORT_PROBE (l->data);
+
+        if (mm_port_probe_is_at (probe) &&
+            g_str_has_prefix (mm_port_probe_get_port_subsys (probe), "usb") &&
+            g_str_has_prefix (mm_port_probe_get_port_name (probe), "cdc-wdm")) {
+            /* Flag as PRIMARY and do nothing else */
+            g_object_set_data (G_OBJECT (probe), TAG_AT_PORT_FLAGS, GUINT_TO_POINTER (MM_PORT_SERIAL_AT_FLAG_PRIMARY));
+            break;
+        }
+    }
 }
 
 static MMBaseModem *
@@ -448,6 +501,17 @@ create_modem (MMPlugin *self,
     }
 #endif
 
+#if defined WITH_MBIM
+    if (mm_port_probe_list_has_mbim_port (probes)) {
+        mm_dbg ("MBIM-powered Huawei modem found...");
+        return MM_BASE_MODEM (mm_broadband_modem_mbim_new (sysfs_path,
+                                                           drivers,
+                                                           mm_plugin_get_name (self),
+                                                           vendor,
+                                                           product));
+    }
+#endif
+
     return MM_BASE_MODEM (mm_broadband_modem_huawei_new (sysfs_path,
                                                          drivers,
                                                          mm_plugin_get_name (self),
@@ -461,21 +525,45 @@ grab_port (MMPlugin *self,
            MMPortProbe *probe,
            GError **error)
 {
-    gchar *str;
-    MMAtPortFlag pflags;
+    MMPortSerialAtFlag pflags = MM_PORT_SERIAL_AT_FLAG_NONE;
+    GUdevDevice *port;
+    MMPortType port_type;
 
-    pflags = (MMAtPortFlag) GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (probe), TAG_AT_PORT_FLAGS));
-    str = mm_at_port_flag_build_string_from_mask (pflags);
-    mm_dbg ("(%s/%s) Port will have AT flags '%s'",
-            mm_port_probe_get_port_subsys (probe),
-            mm_port_probe_get_port_name (probe),
-            str);
-    g_free (str);
+    port_type = mm_port_probe_get_port_type (probe);
+    port = mm_port_probe_peek_port (probe);
+
+    if (g_udev_device_get_property_as_boolean (port, "ID_MM_HUAWEI_AT_PORT")) {
+        mm_dbg ("(%s/%s)' Port flagged as primary",
+                mm_port_probe_get_port_subsys (probe),
+                mm_port_probe_get_port_name (probe));
+        pflags = MM_PORT_SERIAL_AT_FLAG_PRIMARY;
+    } else if (g_udev_device_get_property_as_boolean (port, "ID_MM_HUAWEI_MODEM_PORT")) {
+        mm_dbg ("(%s/%s) Port flagged as PPP",
+                mm_port_probe_get_port_subsys (probe),
+                mm_port_probe_get_port_name (probe));
+        pflags = MM_PORT_SERIAL_AT_FLAG_PPP;
+    } else if (g_udev_device_get_property_as_boolean (port, "ID_MM_HUAWEI_GPS_PORT")) {
+        mm_dbg ("(%s/%s) Port flagged as GPS",
+                mm_port_probe_get_port_subsys (probe),
+                mm_port_probe_get_port_name (probe));
+        port_type = MM_PORT_TYPE_GPS;
+    } else {
+        gchar *str;
+
+        pflags = (MMPortSerialAtFlag) GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (probe), TAG_AT_PORT_FLAGS));
+        str = mm_port_serial_at_flag_build_string_from_mask (pflags);
+        mm_dbg ("(%s/%s) Port will have AT flags '%s'",
+                mm_port_probe_get_port_subsys (probe),
+                mm_port_probe_get_port_name (probe),
+                str);
+        g_free (str);
+    }
 
     return mm_base_modem_grab_port (modem,
                                     mm_port_probe_get_port_subsys (probe),
                                     mm_port_probe_get_port_name (probe),
-                                    mm_port_probe_get_port_type (probe),
+                                    mm_port_probe_get_parent_path (probe),
+                                    port_type,
                                     pflags,
                                     error);
 }
@@ -499,6 +587,8 @@ mm_plugin_create (void)
                       MM_PLUGIN_ALLOWED_VENDOR_IDS, vendor_ids,
                       MM_PLUGIN_ALLOWED_AT,         TRUE,
                       MM_PLUGIN_ALLOWED_QCDM,       TRUE,
+                      MM_PLUGIN_ALLOWED_QMI,        TRUE,
+                      MM_PLUGIN_ALLOWED_MBIM,       TRUE,
                       MM_PLUGIN_CUSTOM_INIT,        &custom_init,
                       NULL));
 }
