@@ -753,7 +753,7 @@ load_current_modes (MMIfaceModem *self,
                     gpointer user_data)
 {
     GSimpleAsyncResult *result;
-    MMAtSerialPort *primary;
+    MMPortSerialAt *primary;
 
     result = g_simple_async_result_new (G_OBJECT (self),
                                         callback,
@@ -830,7 +830,7 @@ set_current_modes (MMIfaceModem *self,
                    gpointer user_data)
 {
     GSimpleAsyncResult *result;
-    MMAtSerialPort *primary;
+    MMPortSerialAt *primary;
     gint idx = -1;
     gchar *command;
 
@@ -1088,15 +1088,15 @@ modem_load_own_numbers (MMIfaceModem *self,
 /*****************************************************************************/
 /* Create Bearer (Modem interface) */
 
-static MMBearer *
+static MMBaseBearer *
 modem_create_bearer_finish (MMIfaceModem *self,
                             GAsyncResult *res,
                             GError **error)
 {
-    MMBearer *bearer;
+    MMBaseBearer *bearer;
 
     bearer = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
-    mm_dbg ("New Sierra bearer created at DBus path '%s'", mm_bearer_get_path (bearer));
+    mm_dbg ("New Sierra bearer created at DBus path '%s'", mm_base_bearer_get_path (bearer));
 
     return g_object_ref (bearer);
 }
@@ -1106,7 +1106,7 @@ broadband_bearer_sierra_new_ready (GObject *source,
                                    GAsyncResult *res,
                                    GSimpleAsyncResult *simple)
 {
-    MMBearer *bearer = NULL;
+    MMBaseBearer *bearer = NULL;
     GError *error = NULL;
 
     bearer = mm_broadband_bearer_sierra_new_finish (res, &error);
@@ -1134,8 +1134,9 @@ modem_create_bearer (MMIfaceModem *self,
                                         modem_create_bearer);
 
     mm_dbg ("Creating Sierra bearer...");
-    mm_broadband_bearer_sierra_new (MM_BROADBAND_MODEM_SIERRA (self),
+    mm_broadband_bearer_sierra_new (MM_BROADBAND_MODEM (self),
                                     properties,
+                                    FALSE, /* is_icera */
                                     NULL, /* cancellable */
                                     (GAsyncReadyCallback)broadband_bearer_sierra_new_ready,
                                     result);
