@@ -26,24 +26,24 @@
 
 #include "ModemManager.h"
 
-#include "mm-manager.h"
+#include "mm-base-manager.h"
 #include "mm-log.h"
 #include "mm-context.h"
 
-#if !defined(MM_DIST_VERSION)
-# define MM_DIST_VERSION VERSION
-#endif
-
 /* Maximum time to wait for all modems to get disabled and removed */
 #define MAX_SHUTDOWN_TIME_SECS 20
 
 static GMainLoop *loop;
-static MMManager *manager;
+static MMBaseManager *manager;
 
 static gboolean
 quit_cb (gpointer user_data)
 {
-	mm_info ("Caught signal, shutting down...");
+    mm_info ("Caught signal, shutting down...");
+
+    if (manager)
+        g_object_set (manager, MM_BASE_MANAGER_CONNECTION, NULL, NULL);
+
     if (loop)
         g_idle_add ((GSourceFunc) g_main_loop_quit, loop);
     else
@@ -62,7 +62,11 @@ bus_acquired_cb (GDBusConnection *connection,
 
     /* Create Manager object */
     g_assert (!manager);
-    manager = mm_manager_new (connection, &error);
+    manager = mm_base_manager_new (connection,
+                                   mm_context_get_test_plugin_dir (),
+                                   !mm_context_get_test_no_auto_scan (),
+                                   mm_context_get_test_enable (),
+                                   &error);
     if (!manager) {
         mm_warn ("Could not create manager: %s", error->message);
         g_error_free (error);
@@ -80,7 +84,7 @@ name_acquired_cb (GDBusConnection *connection,
 
     /* Launch automatic scan for devices */
     g_assert (manager);
-    mm_manager_start (manager, FALSE);
+    mm_base_manager_start (manager, FALSE);
 }
 
 static void
@@ -96,6 +100,9 @@ name_lost_cb (GDBusConnection *connection,
     else
         mm_warn ("Could not acquire the '%s' service name", name);
 
+    if (manager)
+        g_object_set (manager, MM_BASE_MANAGER_CONNECTION, NULL, NULL);
+
     g_main_loop_quit (loop);
 }
 
@@ -125,10 +132,11 @@ main (int argc, char *argv[])
     g_unix_signal_add (SIGTERM, quit_cb, NULL);
     g_unix_signal_add (SIGINT, quit_cb, NULL);
 
-    mm_info ("ModemManager (version " MM_DIST_VERSION ") starting...");
+    mm_info ("ModemManager (version " MM_DIST_VERSION ") starting in %s bus...",
+             mm_context_get_test_session () ? "session" : "system");
 
     /* Acquire name, don't allow replacement */
-    name_id = g_bus_own_name (G_BUS_TYPE_SYSTEM,
+    name_id = g_bus_own_name (mm_context_get_test_session () ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM,
                               MM_DBUS_SERVICE,
                               G_BUS_NAME_OWNER_FLAGS_NONE,
                               bus_acquired_cb,
@@ -149,13 +157,13 @@ main (int argc, char *argv[])
     if (manager) {
         GTimer *timer;
 
-        mm_manager_shutdown (manager);
+        mm_base_manager_shutdown (manager);
 
         /* Wait for all modems to be disabled and removed, but don't wait
          * forever: if disabling the modems takes longer than 20s, just
          * shutdown anyway. */
         timer = g_timer_new ();
-        while (mm_manager_num_modems (manager) &&
+        while (mm_base_manager_num_modems (manager) &&
                g_timer_elapsed (timer, NULL) < (gdouble)MAX_SHUTDOWN_TIME_SECS) {
             GMainContext *ctx = g_main_loop_get_context (inner);
 
@@ -163,10 +171,10 @@ main (int argc, char *argv[])
             g_usleep (50);
         }
 
-        if (mm_manager_num_modems (manager))
+        if (mm_base_manager_num_modems (manager))
             mm_warn ("Disabling modems took too long, "
                      "shutting down with '%u' modems around",
-                     mm_manager_num_modems (manager));
+                     mm_base_manager_num_modems (manager));
 
         g_object_unref (manager);
         g_timer_destroy (timer);
