@@ -227,7 +227,7 @@ typedef struct {
     MmGdbusModemCdma *skeleton;
     GDBusMethodInvocation *invocation;
     MMIfaceModemCdma *self;
-    GVariant *properties;
+    GVariant *dictionary;
 } HandleActivateManualContext;
 
 static void
@@ -236,7 +236,7 @@ handle_activate_manual_context_free (HandleActivateManualContext *ctx)
     g_object_unref (ctx->skeleton);
     g_object_unref (ctx->invocation);
     g_object_unref (ctx->self);
-    g_variant_unref (ctx->properties);
+    g_variant_unref (ctx->dictionary);
     g_free (ctx);
 }
 
@@ -260,6 +260,7 @@ handle_activate_manual_auth_ready (MMBaseModem *self,
                                    GAsyncResult *res,
                                    HandleActivateManualContext *ctx)
 {
+    MMCdmaManualActivationProperties *properties;
     MMModemState modem_state;
     GError *error = NULL;
 
@@ -277,7 +278,7 @@ handle_activate_manual_auth_ready (MMBaseModem *self,
         return;
     }
 
-    /* If activating OTA is not implemented, report an error */
+    /* If manual activation is not implemented, report an error */
     if (!MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->activate_manual ||
         !MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->activate_manual_finish) {
         g_dbus_method_invocation_return_error (ctx->invocation,
@@ -289,6 +290,14 @@ handle_activate_manual_auth_ready (MMBaseModem *self,
         return;
     }
 
+    /* Parse input properties */
+    properties = mm_cdma_manual_activation_properties_new_from_dictionary (ctx->dictionary, &error);
+    if (!properties) {
+        g_dbus_method_invocation_take_error (ctx->invocation, error);
+        handle_activate_manual_context_free (ctx);
+        return;
+    }
+
     modem_state = MM_MODEM_STATE_UNKNOWN;
     g_object_get (self,
                   MM_IFACE_MODEM_STATE, &modem_state,
@@ -308,17 +317,17 @@ handle_activate_manual_auth_ready (MMBaseModem *self,
                                                MM_CORE_ERROR_WRONG_STATE,
                                                "Cannot perform manual activation: "
                                                "device not fully initialized yet");
-        handle_activate_manual_context_free (ctx);
-        return;
+        break;
 
     case MM_MODEM_STATE_ENABLED:
     case MM_MODEM_STATE_SEARCHING:
     case MM_MODEM_STATE_REGISTERED:
         MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->activate_manual (
             MM_IFACE_MODEM_CDMA (self),
-            ctx->properties,
+            properties,
             (GAsyncReadyCallback)handle_activate_manual_ready,
             ctx);
+        g_object_unref (properties);
         return;
 
     case MM_MODEM_STATE_DISABLING:
@@ -349,13 +358,14 @@ handle_activate_manual_auth_ready (MMBaseModem *self,
         break;
     }
 
+    g_object_unref (properties);
     handle_activate_manual_context_free (ctx);
 }
 
 static gboolean
 handle_activate_manual (MmGdbusModemCdma *skeleton,
                         GDBusMethodInvocation *invocation,
-                        GVariant *properties,
+                        GVariant *dictionary,
                         MMIfaceModemCdma *self)
 {
     HandleActivateManualContext *ctx;
@@ -364,7 +374,7 @@ handle_activate_manual (MmGdbusModemCdma *skeleton,
     ctx->skeleton = g_object_ref (skeleton);
     ctx->invocation = g_object_ref (invocation);
     ctx->self = g_object_ref (self);
-    ctx->properties = g_variant_ref (properties);
+    ctx->dictionary = g_variant_ref (dictionary);
 
     mm_base_modem_authorize (MM_BASE_MODEM (self),
                              invocation,
@@ -1404,7 +1414,6 @@ typedef enum {
     ENABLING_STEP_FIRST,
     ENABLING_STEP_SETUP_UNSOLICITED_EVENTS,
     ENABLING_STEP_ENABLE_UNSOLICITED_EVENTS,
-    ENABLING_STEP_RUN_REGISTRATION_CHECKS,
     ENABLING_STEP_PERIODIC_REGISTRATION_CHECKS,
     ENABLING_STEP_LAST
 } EnablingStep;
@@ -1489,24 +1498,6 @@ enable_unsolicited_events_ready (MMIfaceModemCdma *self,
     interface_enabling_step (ctx);
 }
 
-static void
-run_registration_checks_ready (MMIfaceModemCdma *self,
-                                   GAsyncResult *res,
-                                   EnablingContext *ctx)
-{
-    GError *error = NULL;
-
-    if (!mm_iface_modem_cdma_run_registration_checks_finish (self, res, &error)) {
-        g_simple_async_result_take_error (ctx->result, error);
-        enabling_context_complete_and_free (ctx);
-        return;
-    }
-
-    /* Go on to next step */
-    ctx->step++;
-    interface_enabling_step (ctx);
-}
-
 static void
 interface_enabling_step (EnablingContext *ctx)
 {
@@ -1543,12 +1534,6 @@ interface_enabling_step (EnablingContext *ctx)
         /* Fall down to next step */
         ctx->step++;
 
-    case ENABLING_STEP_RUN_REGISTRATION_CHECKS:
-        mm_iface_modem_cdma_run_registration_checks (ctx->self,
-                                                     (GAsyncReadyCallback)run_registration_checks_ready,
-                                                     ctx);
-        return;
-
     case ENABLING_STEP_PERIODIC_REGISTRATION_CHECKS:
         periodic_registration_check_enable (ctx->self);
         /* Fall down to next step */
