@@ -30,10 +30,9 @@
 
 #include "mm-plugin.h"
 #include "mm-device.h"
-#include "mm-at-serial-port.h"
-#include "mm-qcdm-serial-port.h"
+#include "mm-port-serial-at.h"
+#include "mm-port-serial-qcdm.h"
 #include "mm-serial-parsers.h"
-#include "mm-marshal.h"
 #include "mm-private-boxed-types.h"
 #include "mm-log.h"
 #include "mm-daemon-enums-types.h"
@@ -311,12 +310,17 @@ apply_pre_probing_filters (MMPlugin *self,
         }
     }
 
-    /* If we got filtered by vendor or product IDs  and we do not have vendor
-     * or product strings to compare with: unsupported */
+    /* If we got filtered by vendor or product IDs; mark it as unsupported only if:
+     *   a) we do not have vendor or product strings to compare with (i.e. plugin
+     *      doesn't have explicit vendor/product strings
+     *   b) the port is NOT an AT port which we can use for AT probing
+     */
     if ((vendor_filtered || product_filtered) &&
-        !self->priv->vendor_strings &&
-        !self->priv->product_strings &&
-        !self->priv->forbidden_product_strings) {
+        ((!self->priv->vendor_strings &&
+          !self->priv->product_strings &&
+          !self->priv->forbidden_product_strings) ||
+         g_str_equal (g_udev_device_get_subsystem (port), "net") ||
+         g_str_has_prefix (g_udev_device_get_name (port), "cdc-wdm"))) {
         mm_dbg ("(%s) [%s] filtered by vendor/product IDs",
                 self->priv->name,
                 g_udev_device_get_name (port));
@@ -628,21 +632,6 @@ mm_plugin_supports_port_finish (MMPlugin *self,
     return (MMPluginSupportsResult) GPOINTER_TO_UINT (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (result)));
 }
 
-static gboolean
-find_driver_in_device (MMDevice *device,
-                       const gchar *driver)
-{
-    const gchar **device_drivers;
-    guint i;
-
-    device_drivers = mm_device_get_drivers (device);
-    for (i = 0; device_drivers[i]; i++) {
-        if (g_str_equal (driver, device_drivers[i]))
-            return TRUE;
-    }
-    return FALSE;
-}
-
 void
 mm_plugin_supports_port (MMPlugin *self,
                          MMDevice *device,
@@ -714,21 +703,27 @@ mm_plugin_supports_port (MMPlugin *self,
             probe_run_flags |= MM_PORT_PROBE_AT;
         else if (self->priv->single_at)
             probe_run_flags |= MM_PORT_PROBE_AT;
-        if (need_vendor_probing)
-            probe_run_flags |= (MM_PORT_PROBE_AT | MM_PORT_PROBE_AT_VENDOR);
-        if (need_product_probing)
-            probe_run_flags |= (MM_PORT_PROBE_AT | MM_PORT_PROBE_AT_PRODUCT);
         if (self->priv->qcdm)
             probe_run_flags |= MM_PORT_PROBE_QCDM;
-        if (self->priv->icera_probe || self->priv->allowed_icera || self->priv->forbidden_icera)
-            probe_run_flags |= (MM_PORT_PROBE_AT | MM_PORT_PROBE_AT_ICERA);
     } else {
         /* cdc-wdm ports... */
         probe_run_flags = MM_PORT_PROBE_NONE;
-        if (self->priv->qmi && find_driver_in_device (device, "qmi_wwan"))
+        if (self->priv->qmi && !g_strcmp0 (mm_device_utils_get_port_driver (port), "qmi_wwan"))
             probe_run_flags |= MM_PORT_PROBE_QMI;
-        if (self->priv->mbim && find_driver_in_device (device, "cdc_mbim"))
+        else if (self->priv->mbim && !g_strcmp0 (mm_device_utils_get_port_driver (port), "cdc_mbim"))
             probe_run_flags |= MM_PORT_PROBE_MBIM;
+        else
+            probe_run_flags |= MM_PORT_PROBE_AT;
+    }
+
+    /* For potential AT ports, check for more things */
+    if (probe_run_flags & MM_PORT_PROBE_AT) {
+        if (need_vendor_probing)
+            probe_run_flags |= MM_PORT_PROBE_AT_VENDOR;
+        if (need_product_probing)
+            probe_run_flags |= MM_PORT_PROBE_AT_PRODUCT;
+        if (self->priv->icera_probe || self->priv->allowed_icera || self->priv->forbidden_icera)
+            probe_run_flags |= MM_PORT_PROBE_AT_ICERA;
     }
 
     /* If no explicit probing was required, just request to grab it without probing anything.
@@ -826,10 +821,14 @@ mm_plugin_create_modem (MMPlugin  *self,
                         MMDevice *device,
                         GError   **error)
 {
-    MMBaseModem *modem = NULL;
-    GList *port_probes, *l;
+    MMBaseModem *modem;
+    GList *port_probes = NULL;
+    const gchar **virtual_ports = NULL;
 
-    port_probes = mm_device_peek_port_probe_list (device);
+    if (!mm_device_is_virtual (device))
+        port_probes = mm_device_peek_port_probe_list (device);
+    else
+        virtual_ports = mm_device_virtual_peek_ports (device);
 
     /* Let the plugin create the modem from the port probe results */
     modem = MM_PLUGIN_GET_CLASS (self)->create_modem (MM_PLUGIN (self),
@@ -839,8 +838,13 @@ mm_plugin_create_modem (MMPlugin  *self,
                                                       mm_device_get_product (device),
                                                       port_probes,
                                                       error);
-    if (modem) {
-        mm_base_modem_set_hotplugged (modem, mm_device_get_hotplugged (device));
+    if (!modem)
+        return NULL;
+
+    mm_base_modem_set_hotplugged (modem, mm_device_get_hotplugged (device));
+
+    if (port_probes) {
+        GList *l;
 
         /* Grab each port */
         for (l = port_probes; l; l = g_list_next (l)) {
@@ -863,8 +867,7 @@ mm_plugin_create_modem (MMPlugin  *self,
             }
 #if !defined WITH_QMI
             else if (mm_port_probe_get_port_type (probe) == MM_PORT_TYPE_NET &&
-                     g_str_equal (mm_device_utils_get_port_driver (mm_port_probe_peek_port (probe)),
-                                  "qmi_wwan")) {
+                     !g_strcmp0 (mm_device_utils_get_port_driver (mm_port_probe_peek_port (probe)), "qmi_wwan")) {
                 grabbed = FALSE;
                 inner_error = g_error_new (MM_CORE_ERROR,
                                            MM_CORE_ERROR_UNSUPPORTED,
@@ -873,8 +876,7 @@ mm_plugin_create_modem (MMPlugin  *self,
 #endif
 #if !defined WITH_MBIM
             else if (mm_port_probe_get_port_type (probe) == MM_PORT_TYPE_NET &&
-                     g_str_equal (mm_device_utils_get_port_driver (mm_port_probe_peek_port (probe)),
-                                  "cdc_mbim")) {
+                     !g_strcmp0 (mm_device_utils_get_port_driver (mm_port_probe_peek_port (probe)), "cdc_mbim")) {
                 grabbed = FALSE;
                 inner_error = g_error_new (MM_CORE_ERROR,
                                            MM_CORE_ERROR_UNSUPPORTED,
@@ -890,8 +892,9 @@ mm_plugin_create_modem (MMPlugin  *self,
                 grabbed = mm_base_modem_grab_port (modem,
                                                    mm_port_probe_get_port_subsys (probe),
                                                    mm_port_probe_get_port_name (probe),
+                                                   mm_port_probe_get_parent_path (probe),
                                                    mm_port_probe_get_port_type (probe),
-                                                   MM_AT_PORT_FLAG_NONE,
+                                                   MM_PORT_SERIAL_AT_FLAG_NONE,
                                                    &inner_error);
             if (!grabbed) {
                 mm_warn ("Could not grab port (%s/%s): '%s'",
@@ -901,12 +904,31 @@ mm_plugin_create_modem (MMPlugin  *self,
                 g_clear_error (&inner_error);
             }
         }
+    } else if (virtual_ports) {
+        guint i;
 
-        /* If organizing ports fails, consider the modem invalid */
-        if (!mm_base_modem_organize_ports (modem, error))
-            g_clear_object (&modem);
+        for (i = 0; virtual_ports[i]; i++) {
+            GError *inner_error = NULL;
+
+            if (!mm_base_modem_grab_port (modem,
+                                          "virtual",
+                                          virtual_ports[i],
+                                          NULL,
+                                          MM_PORT_TYPE_AT,
+                                          MM_PORT_SERIAL_AT_FLAG_NONE,
+                                          &inner_error)) {
+                mm_warn ("Could not grab port (virtual/%s): '%s'",
+                         virtual_ports[i],
+                         inner_error ? inner_error->message : "unknown error");
+                g_clear_error (&inner_error);
+            }
+        }
     }
 
+    /* If organizing ports fails, consider the modem invalid */
+    if (!mm_base_modem_organize_ports (modem, error))
+        g_clear_object (&modem);
+
     return modem;
 }
 
