@@ -27,7 +27,7 @@
 #include "mm-log.h"
 #include "mm-sim-qmi.h"
 
-G_DEFINE_TYPE (MMSimQmi, mm_sim_qmi, MM_TYPE_SIM);
+G_DEFINE_TYPE (MMSimQmi, mm_sim_qmi, MM_TYPE_BASE_SIM)
 
 /*****************************************************************************/
 
@@ -40,10 +40,10 @@ ensure_qmi_client (MMSimQmi *self,
 {
     MMBaseModem *modem = NULL;
     QmiClient *client;
-    MMQmiPort *port;
+    MMPortQmi *port;
 
     g_object_get (self,
-                  MM_SIM_MODEM, &modem,
+                  MM_BASE_SIM_MODEM, &modem,
                   NULL);
     g_assert (MM_IS_BASE_MODEM (modem));
 
@@ -60,9 +60,9 @@ ensure_qmi_client (MMSimQmi *self,
         return FALSE;
     }
 
-    client = mm_qmi_port_peek_client (port,
+    client = mm_port_qmi_peek_client (port,
                                       service,
-                                      MM_QMI_PORT_FLAG_DEFAULT);
+                                      MM_PORT_QMI_FLAG_DEFAULT);
     if (!client) {
         g_simple_async_report_error_in_idle (G_OBJECT (self),
                                              callback,
@@ -82,7 +82,7 @@ ensure_qmi_client (MMSimQmi *self,
 /* Load SIM ID (ICCID) */
 
 static gchar *
-load_sim_identifier_finish (MMSim *self,
+load_sim_identifier_finish (MMBaseSim *self,
                             GAsyncResult *res,
                             GError **error)
 {
@@ -128,7 +128,7 @@ dms_uim_get_iccid_ready (QmiClientDms *client,
 }
 
 static void
-load_sim_identifier (MMSim *self,
+load_sim_identifier (MMBaseSim *self,
                      GAsyncReadyCallback callback,
                      gpointer user_data)
 {
@@ -158,7 +158,7 @@ load_sim_identifier (MMSim *self,
 /* Load IMSI */
 
 static gchar *
-load_imsi_finish (MMSim *self,
+load_imsi_finish (MMBaseSim *self,
                   GAsyncResult *res,
                   GError **error)
 {
@@ -204,7 +204,7 @@ dms_uim_get_imsi_ready (QmiClientDms *client,
 }
 
 static void
-load_imsi (MMSim *self,
+load_imsi (MMBaseSim *self,
            GAsyncReadyCallback callback,
            gpointer user_data)
 {
@@ -230,6 +230,182 @@ load_imsi (MMSim *self,
                                  result);
 }
 
+/*****************************************************************************/
+/* Load operator identifier */
+
+static gboolean
+get_home_network (QmiClientNas *client,
+                  GAsyncResult *res,
+                  guint16      *out_mcc,
+                  guint16      *out_mnc,
+                  gchar        **out_operator_name,
+                  GError       **error)
+{
+    QmiMessageNasGetHomeNetworkOutput *output = NULL;
+    gboolean success = FALSE;
+
+    output = qmi_client_nas_get_home_network_finish (client, res, error);
+    if (!output) {
+        g_prefix_error (error, "QMI operation failed: ");
+    } else if (!qmi_message_nas_get_home_network_output_get_result (output, error)) {
+        g_prefix_error (error, "Couldn't get home network: ");
+    } else {
+        const gchar *name = NULL;
+
+        qmi_message_nas_get_home_network_output_get_home_network (
+            output,
+            out_mcc,
+            out_mnc,
+            &name,
+            NULL);
+        if (out_operator_name)
+            *out_operator_name = g_strdup (name);
+        success = TRUE;
+    }
+
+    if (output)
+        qmi_message_nas_get_home_network_output_unref (output);
+
+    return success;
+}
+
+static gchar *
+load_operator_identifier_finish (MMBaseSim *self,
+                                 GAsyncResult *res,
+                                 GError **error)
+{
+    gchar *operator_identifier;
+
+    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+        return NULL;
+
+    operator_identifier = g_strdup (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
+    mm_dbg ("loaded operator identifier: %s", operator_identifier);
+    return operator_identifier;
+}
+
+static void
+load_operator_identifier_ready (QmiClientNas *client,
+                                GAsyncResult *res,
+                                GSimpleAsyncResult *simple)
+{
+    guint16 mcc, mnc;
+    GError *error = NULL;
+
+    if (get_home_network (client, res, &mcc, &mnc, NULL, &error)) {
+        GString *aux;
+
+        aux = g_string_new ("");
+        /* MCC always 3 digits */
+        g_string_append_printf (aux, "%.3" G_GUINT16_FORMAT, mcc);
+        /* Guess about MNC, if < 100 assume it's 2 digits, no PCS info here */
+        if (mnc >= 100)
+            g_string_append_printf (aux, "%.3" G_GUINT16_FORMAT, mnc);
+        else
+            g_string_append_printf (aux, "%.2" G_GUINT16_FORMAT, mnc);
+        g_simple_async_result_set_op_res_gpointer (simple,
+                                                   g_string_free (aux, FALSE),
+                                                   (GDestroyNotify)g_free);
+    } else {
+        g_simple_async_result_take_error (simple, error);
+    }
+
+    g_simple_async_result_complete (simple);
+    g_object_unref (simple);
+}
+
+static void
+load_operator_identifier (MMBaseSim *self,
+                          GAsyncReadyCallback callback,
+                          gpointer user_data)
+{
+    GSimpleAsyncResult *result;
+    QmiClient *client = NULL;
+
+    if (!ensure_qmi_client (MM_SIM_QMI (self),
+                            QMI_SERVICE_NAS, &client,
+                            callback, user_data))
+        return;
+
+    result = g_simple_async_result_new (G_OBJECT (self),
+                                        callback,
+                                        user_data,
+                                        load_operator_identifier);
+
+    mm_dbg ("loading SIM operator identifier...");
+    qmi_client_nas_get_home_network (QMI_CLIENT_NAS (client),
+                                     NULL,
+                                     5,
+                                     NULL,
+                                     (GAsyncReadyCallback)load_operator_identifier_ready,
+                                     result);
+}
+
+/*****************************************************************************/
+/* Load operator name */
+
+static gchar *
+load_operator_name_finish (MMBaseSim *self,
+                           GAsyncResult *res,
+                           GError **error)
+{
+    gchar *operator_name;
+
+    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+        return NULL;
+
+    operator_name = g_strdup (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
+    mm_dbg ("loaded operator name: %s", operator_name);
+    return operator_name;
+}
+
+static void
+load_operator_name_ready (QmiClientNas *client,
+                          GAsyncResult *res,
+                          GSimpleAsyncResult *simple)
+{
+    gchar *operator_name = NULL;
+    GError *error = NULL;
+
+    if (get_home_network (client, res, NULL, NULL, &operator_name, &error)) {
+        g_simple_async_result_set_op_res_gpointer (simple,
+                                                   operator_name,
+                                                   (GDestroyNotify)g_free);
+    } else {
+        g_simple_async_result_take_error (simple, error);
+    }
+
+    g_simple_async_result_complete (simple);
+    g_object_unref (simple);
+}
+
+static void
+load_operator_name (MMBaseSim *self,
+                    GAsyncReadyCallback callback,
+                    gpointer user_data)
+{
+    GSimpleAsyncResult *result;
+    QmiClient *client = NULL;
+
+    if (!ensure_qmi_client (MM_SIM_QMI (self),
+                            QMI_SERVICE_NAS, &client,
+                            callback, user_data))
+        return;
+
+    result = g_simple_async_result_new (G_OBJECT (self),
+                                        callback,
+                                        user_data,
+                                        load_operator_name);
+
+    mm_dbg ("loading SIM operator name...");
+    qmi_client_nas_get_home_network (QMI_CLIENT_NAS (client),
+                                     NULL,
+                                     5,
+                                     NULL,
+                                     (GAsyncReadyCallback)load_operator_name_ready,
+                                     result);
+}
+
 /*****************************************************************************/
 /* Send PIN */
 
@@ -261,7 +437,7 @@ pin_qmi_error_to_mobile_equipment_error (GError *qmi_error)
 }
 
 static gboolean
-send_pin_finish (MMSim *self,
+send_pin_finish (MMBaseSim *self,
                  GAsyncResult *res,
                  GError **error)
 {
@@ -296,7 +472,7 @@ dms_uim_verify_pin_ready (QmiClientDms *client,
 }
 
 static void
-send_pin (MMSim *self,
+send_pin (MMBaseSim *self,
           const gchar *pin,
           GAsyncReadyCallback callback,
           gpointer user_data)
@@ -335,7 +511,7 @@ send_pin (MMSim *self,
 /* Send PUK */
 
 static gboolean
-send_puk_finish (MMSim *self,
+send_puk_finish (MMBaseSim *self,
                  GAsyncResult *res,
                  GError **error)
 {
@@ -370,7 +546,7 @@ dms_uim_unblock_pin_ready (QmiClientDms *client,
 }
 
 static void
-send_puk (MMSim *self,
+send_puk (MMBaseSim *self,
           const gchar *puk,
           const gchar *new_pin,
           GAsyncReadyCallback callback,
@@ -412,7 +588,7 @@ send_puk (MMSim *self,
 /* Change PIN */
 
 static gboolean
-change_pin_finish (MMSim *self,
+change_pin_finish (MMBaseSim *self,
                    GAsyncResult *res,
                    GError **error)
 {
@@ -447,7 +623,7 @@ dms_uim_change_pin_ready (QmiClientDms *client,
 }
 
 static void
-change_pin (MMSim *self,
+change_pin (MMBaseSim *self,
             const gchar *old_pin,
             const gchar *new_pin,
             GAsyncReadyCallback callback,
@@ -489,7 +665,7 @@ change_pin (MMSim *self,
 /* Enable PIN */
 
 static gboolean
-enable_pin_finish (MMSim *self,
+enable_pin_finish (MMBaseSim *self,
                    GAsyncResult *res,
                    GError **error)
 {
@@ -524,7 +700,7 @@ dms_uim_set_pin_protection_ready (QmiClientDms *client,
 }
 
 static void
-enable_pin (MMSim *self,
+enable_pin (MMBaseSim *self,
             const gchar *pin,
             gboolean enabled,
             GAsyncReadyCallback callback,
@@ -565,7 +741,7 @@ enable_pin (MMSim *self,
 
 /*****************************************************************************/
 
-MMSim *
+MMBaseSim *
 mm_sim_qmi_new_finish (GAsyncResult  *res,
                        GError       **error)
 {
@@ -580,9 +756,9 @@ mm_sim_qmi_new_finish (GAsyncResult  *res,
         return NULL;
 
     /* Only export valid SIMs */
-    mm_sim_export (MM_SIM (sim));
+    mm_base_sim_export (MM_BASE_SIM (sim));
 
-    return MM_SIM (sim);
+    return MM_BASE_SIM (sim);
 }
 
 void
@@ -596,7 +772,7 @@ mm_sim_qmi_new (MMBaseModem *modem,
                                 cancellable,
                                 callback,
                                 user_data,
-                                MM_SIM_MODEM, modem,
+                                MM_BASE_SIM_MODEM, modem,
                                 NULL);
 }
 
@@ -608,22 +784,22 @@ mm_sim_qmi_init (MMSimQmi *self)
 static void
 mm_sim_qmi_class_init (MMSimQmiClass *klass)
 {
-    MMSimClass *sim_class = MM_SIM_CLASS (klass);
+    MMBaseSimClass *base_sim_class = MM_BASE_SIM_CLASS (klass);
 
-    sim_class->load_sim_identifier = load_sim_identifier;
-    sim_class->load_sim_identifier_finish = load_sim_identifier_finish;
-    sim_class->load_imsi = load_imsi;
-    sim_class->load_imsi_finish = load_imsi_finish;
-    sim_class->load_operator_identifier = NULL;
-    sim_class->load_operator_identifier_finish = NULL;
-    sim_class->load_operator_name = NULL;
-    sim_class->load_operator_name_finish = NULL;
-    sim_class->send_pin = send_pin;
-    sim_class->send_pin_finish = send_pin_finish;
-    sim_class->send_puk = send_puk;
-    sim_class->send_puk_finish = send_puk_finish;
-    sim_class->change_pin = change_pin;
-    sim_class->change_pin_finish = change_pin_finish;
-    sim_class->enable_pin = enable_pin;
-    sim_class->enable_pin_finish = enable_pin_finish;
+    base_sim_class->load_sim_identifier = load_sim_identifier;
+    base_sim_class->load_sim_identifier_finish = load_sim_identifier_finish;
+    base_sim_class->load_imsi = load_imsi;
+    base_sim_class->load_imsi_finish = load_imsi_finish;
+    base_sim_class->load_operator_identifier = load_operator_identifier;
+    base_sim_class->load_operator_identifier_finish = load_operator_identifier_finish;
+    base_sim_class->load_operator_name = load_operator_name;
+    base_sim_class->load_operator_name_finish = load_operator_name_finish;
+    base_sim_class->send_pin = send_pin;
+    base_sim_class->send_pin_finish = send_pin_finish;
+    base_sim_class->send_puk = send_puk;
+    base_sim_class->send_puk_finish = send_puk_finish;
+    base_sim_class->change_pin = change_pin;
+    base_sim_class->change_pin_finish = change_pin_finish;
+    base_sim_class->enable_pin = enable_pin;
+    base_sim_class->enable_pin_finish = enable_pin_finish;
 }
